using System;
using System.IO;
using System.Text;

class MyApp
{
    const int bufsize = 1024;

    static void Main (string[] args)
    {
        // Make sure a file name and encryption key were entered
        if (args.Length < 2) {
            Console.WriteLine ("Syntax: SCRAMBLE filename key");
            return;
        }

        string filename = args[0];
        string key = args[1];
        FileStream stream = null;

        try {
            // Open the file for reading and writing
            stream = File.Open (filename, FileMode.Open,
                FileAccess.ReadWrite);

            // Wrap a reader and writer around the FileStream
            BinaryReader reader = new BinaryReader (stream);
            BinaryWriter writer = new BinaryWriter (stream);

            // Convert the key into a byte array
            ASCIIEncoding enc = new ASCIIEncoding ();
            byte[] keybytes = enc.GetBytes (key);

            // Allocate an I/O buffer and a key buffer
            byte[] buffer = new byte[bufsize];
            byte[] keybuf = new byte[bufsize + keybytes.Length - 1];

            // Replicate the byte array in the key buffer to create
            // an encryption key whose size equals or exceeds the
            // size of the I/O buffer
            int count = (1024 + keybytes.Length - 1) / keybytes.Length;
            for (int i=0; i<count; i++)            
                Array.Copy (keybytes, 0, keybuf, i * keybytes.Length,
                    keybytes.Length);

            // Read the file in bufsize blocks, XOR-encrypt each block,
            // and write the encrypted block back to the file
            long lBytesRemaining = stream.Length;

            while (lBytesRemaining > 0) {
                long lPosition = stream.Position;
                int nBytesRequested = (int) System.Math.Min (bufsize,
                    lBytesRemaining);
                int nBytesRead = reader.Read (buffer, 0,
                    nBytesRequested);

                for (int i=0; i<nBytesRead; i++)
                    buffer[i] ^= keybuf[i];				

                stream.Seek (lPosition, SeekOrigin.Begin);
                writer.Write (buffer, 0, nBytesRead);
                lBytesRemaining -= nBytesRead;
            }
        }
        catch (Exception e) {
            Console.WriteLine (e.Message);
        }
        finally {
            if (stream != null)
                stream.Close ();
        }
    }
}
