using System;
using System.Windows.Forms;
using System.Drawing;

class MyForm : Form
{
    enum Units {
        Inches,
        Centimeters,
        Pixels
    }

    int MyWidth = 400;
    int MyHeight = 200;
    Units MyUnits = Units.Pixels;

    MyForm ()
    {
        // Set the form's title
        Text = "Dialog Demo";

        // Set the form's size
        ClientSize = new Size (640, 480);

        // Create a menu
        MainMenu menu = new MainMenu ();
        MenuItem item = menu.MenuItems.Add ("&Options");
        item.MenuItems.Add ("Edit Ellipse...",
            new EventHandler (OnEditEllipse));
        item.MenuItems.Add ("-");
        item.MenuItems.Add ("E&xit", new EventHandler (OnExit));

        // Attach the menu to the form
        Menu = menu;
    }

    // Handler for the Edit Ellipse command
    void OnEditEllipse (object sender, EventArgs e)
    {
        // Create the dialog
        MyDialog dlg = new MyDialog ();

        // Initialize the dialog
        dlg.UserWidth = MyWidth;
        dlg.UserHeight = MyHeight;
        dlg.UserUnits = (int) MyUnits;

        // Display the dialog
        if (dlg.ShowDialog (this) == DialogResult.OK) {
            // If the dialog was dismissed with the OK button,
            // extract the user input and repaint the form
            MyWidth = dlg.UserWidth;
            MyHeight = dlg.UserHeight;
            MyUnits = (MyForm.Units) dlg.UserUnits;
            Invalidate ();
        }

        // Dispose of the dialog
        dlg.Dispose ();
    }

    // Handler for the Exit command
    void OnExit (object sender, EventArgs e)
    {
        Close ();
    }

    // OnPaint handler
    protected override void OnPaint (PaintEventArgs e)
    {
        int multiplier = 1;

        switch (MyUnits) {

        case Units.Inches:
            e.Graphics.PageUnit = GraphicsUnit.Inch;
            break;

        case Units.Centimeters:
            e.Graphics.PageUnit = GraphicsUnit.Millimeter;
            multiplier = 10;
            break;

        case Units.Pixels:
            e.Graphics.PageUnit = GraphicsUnit.Pixel;
            break;
        }

        SolidBrush brush = new SolidBrush (Color.Magenta);
        e.Graphics.FillEllipse (brush, 0, 0, MyWidth * multiplier,
            MyHeight * multiplier);
        brush.Dispose ();
    }

    static void Main () 
    {
        Application.Run (new MyForm ());
    }
}

class MyDialog : Form
{
    Label WidthLabel;
    Label HeightLabel;
    TextBox WidthBox;
    TextBox HeightBox;
    GroupBox UnitsGroup;
    RadioButton InchesButton;
    RadioButton CentimetersButton;
    RadioButton PixelsButton;
    Button OKButton;
    Button NotOKButton;

    public int UserWidth
    {
        get { return Convert.ToInt32 (WidthBox.Text); }
        set { WidthBox.Text = value.ToString (); }
    }

    public int UserHeight
    {
        get { return Convert.ToInt32 (HeightBox.Text); }
        set { HeightBox.Text = value.ToString (); }
    }

    public int UserUnits
    {
        get
        {
            for (int i=0; i<UnitsGroup.Controls.Count; i++) {
                RadioButton button =
                    (RadioButton) UnitsGroup.Controls[i];
                if (button.Checked)
                    return i;                
            }
            return -1;
        }
        set
        {
            RadioButton button =
                (RadioButton) UnitsGroup.Controls[value];
            button.Checked = true;
        }
    }

    public MyDialog ()
    {
        // Initialize the dialog's visual properties
        ClientSize = new Size (296, 196);
        StartPosition = FormStartPosition.CenterParent;
        FormBorderStyle = FormBorderStyle.FixedDialog;
        MaximizeBox = false;
        MinimizeBox = false;
        Text = "Edit Ellipse";
        ShowInTaskbar = false;

        // Create the dialog's controls
        WidthLabel = new Label ();
        WidthLabel.Location = new Point (16, 16);
        WidthLabel.Size = new Size (48, 24);
        WidthLabel.Text = "Width";

        HeightLabel = new Label ();
        HeightLabel.Location = new Point (16, 48);
        HeightLabel.Size = new Size (48, 24);
        HeightLabel.Text = "Height";

        WidthBox = new TextBox ();
        WidthBox.Location = new Point (64, 12);
        WidthBox.Size = new Size (96, 24);
        WidthBox.TabIndex = 1;

        HeightBox = new TextBox ();
        HeightBox.Location = new Point (64, 44);
        HeightBox.Size = new Size (96, 24);
        HeightBox.TabIndex = 2;

        UnitsGroup = new GroupBox ();
        UnitsGroup.Location = new Point (16, 76);
        UnitsGroup.Size = new Size (144, 100);
        UnitsGroup.Text = "Units";

        InchesButton = new RadioButton ();
        InchesButton.Location = new Point (16, 24);
        InchesButton.Size = new Size (112, 16);
        InchesButton.Text = "Inches";

        CentimetersButton = new RadioButton ();
        CentimetersButton.Location = new Point (16, 48);
        CentimetersButton.Size = new Size (112, 16);
        CentimetersButton.Text = "Centimeters";

        PixelsButton = new RadioButton ();
        PixelsButton.Location = new Point (16, 72);
        PixelsButton.Size = new Size (112, 16);
        PixelsButton.Text = "Pixels";

        OKButton = new Button ();
        OKButton.Location = new Point (184, 12);
        OKButton.Size = new Size (96, 24);
        OKButton.TabIndex = 3;
        OKButton.Text = "OK";
        OKButton.DialogResult = DialogResult.OK;

        NotOKButton = new Button ();
        NotOKButton.Location = new Point (184, 44);
        NotOKButton.Size = new Size (96, 24);
        NotOKButton.TabIndex = 4;
        NotOKButton.Text = "Cancel";
        NotOKButton.DialogResult = DialogResult.Cancel;

        AcceptButton = OKButton;
        CancelButton = NotOKButton;

        // Add the controls to the dialog
        Controls.Add (WidthLabel);
        Controls.Add (HeightLabel);
        Controls.Add (WidthBox);
        Controls.Add (HeightBox);
        Controls.Add (UnitsGroup);
        UnitsGroup.Controls.Add (InchesButton);
        UnitsGroup.Controls.Add (CentimetersButton);
        UnitsGroup.Controls.Add (PixelsButton);
        Controls.Add (OKButton);
        Controls.Add (NotOKButton);
    }
}