<%@ WebHandler Language="C#" Class="CityViewImageGen" %>

using System;
using System.Web;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;

public class CityViewImageGen : IHttpHandler
{
    public void ProcessRequest (HttpContext context)
    {
        // Extract user input from the query string
        string city = context.Request["City"];
        string state = context.Request["State"];
        string scale = context.Request["Scale"];

        if (city != null && state != null) {
            // Determine the scale
            TS.Scale res = TS.Scale.Scale8m;

            if (scale!= null) {
                switch (scale) {
                case "1":
                    res = TS.Scale.Scale1m;
                    break;

                case "2":
                    res = TS.Scale.Scale2m;
                    break;

                case "4":
                    res = TS.Scale.Scale4m;
                    break;

                case "8":
                    res = TS.Scale.Scale8m;
                    break;

                case "16":
                    res = TS.Scale.Scale16m;
                    break;

                case "32":
                    res = TS.Scale.Scale32m;
                    break;
                }
            }

            // Generate the requested image
            string type = "image/jpeg";
            ImageFormat format = ImageFormat.Jpeg;
            Bitmap bitmap = GetTiledImage (city, state, res, 600, 400);

            // If GetTiledImage failed, generate an error bitmap
            if (bitmap == null) {
                bitmap = GetErrorImage ("Image not available");
                type = "image/gif";
                format = ImageFormat.Gif;
            }

            // Set the responses content type
            context.Response.ContentType = type;

            // Write the image to the HTTP response
            bitmap.Save (context.Response.OutputStream, format);

            // Clean up and return
            bitmap.Dispose ();
        }
    }

    public bool IsReusable
    {
        get { return true; }
    }

    Bitmap GetTiledImage (string City, string State, TS.Scale Scale,
        int cx, int cy)
    {
        Bitmap bitmap = null;
        Graphics g = null;

        try {
            // Instantiate the TerraService proxy
            TS.TerraService ts = new TS.TerraService ();

            // Get the latitude and longitude of the requested city
            TS.Place place = new TS.Place ();
            place.City = City;
            place.State = State;
            place.Country = "USA";
            TS.LonLatPt point = ts.ConvertPlaceToLonLatPt (place);

            // Compute the parameters for a bounding box
            TS.AreaBoundingBox abb = ts.GetAreaFromPt (point,
                TS.Theme.Photo, Scale, cx, cy);

            // Create an image to fit the bounding box
            bitmap = new Bitmap (cx, cy,
                PixelFormat.Format32bppRgb);
            g = Graphics.FromImage (bitmap);

            int x1 = abb.NorthWest.TileMeta.Id.X;
            int y1 = abb.NorthWest.TileMeta.Id.Y;
            int x2 = abb.NorthEast.TileMeta.Id.X;
            int y2 = abb.SouthWest.TileMeta.Id.Y;

            for (int x=x1; x<=x2; x++) {
                for (int y=y1; y>=y2; y--) {
                    TS.TileId tid = abb.NorthWest.TileMeta.Id;
                    tid.X = x;
                    tid.Y = y;
                    Image tile = Image.FromStream
                        (new MemoryStream (ts.GetTile (tid)));
                    g.DrawImage (tile,
                        (x - x1) * tile.Width  -
                        (int) abb.NorthWest.Offset.XOffset,
                        (y1 - y) * tile.Height -
                        (int)abb.NorthWest.Offset.YOffset,
                        tile.Width, tile.Height);
                    tile.Dispose();
                }
            }

            // Return the image
            return bitmap;
        }
        catch (Exception) {
            if (bitmap != null)
                bitmap.Dispose ();
            return null;
        }
        finally {
            if (g != null)
                g.Dispose ();
        }
    }

    Bitmap GetErrorImage (string message)
    {
        // Determine the width and height of the error message
        Bitmap bitmap =
            new Bitmap (1, 1, PixelFormat.Format32bppRgb);
        Graphics g = Graphics.FromImage (bitmap);
        Font font = new Font ("Verdana", 10);
        SizeF size = g.MeasureString (message, font);
        int cx = (int) size.Width;
        int cy = (int) size.Height;
        bitmap.Dispose ();
        g.Dispose ();

        // Generate a bitmap containing the error message
        bitmap = new Bitmap (cx, cy, PixelFormat.Format32bppRgb);
        g = Graphics.FromImage (bitmap);
        SolidBrush redBrush = new SolidBrush (Color.Red);
        SolidBrush whiteBrush = new SolidBrush (Color.White);

        g.FillRectangle (whiteBrush, 0, 0, 256, 64);
        g.DrawString (message, font, redBrush, 0, 0);

        whiteBrush.Dispose ();
        redBrush.Dispose ();
        font.Dispose ();
        g.Dispose ();

        // Return the image
        return bitmap;
    }
}