using System;
using System.Drawing;
using System.Windows.Forms;
using System.Xml.XPath;

class AnalyzerForm : Form
{
    GroupBox DocumentGB;
    TextBox Source;
    Button LoadButton;
    GroupBox ExpressionGB;
    TextBox Expression;
    Button ExecuteButton;
    ImageList NodeImages;
    TreeView XmlView;

    XPathNavigator Navigator;

    AnalyzerForm ()
    {
        // Initialize the form's properties
        Text = "XPath Expression Analyzer";
        ClientSize = new System.Drawing.Size (488, 422);

        // Instantiate the form's controls
        DocumentGB = new GroupBox ();
        Source = new TextBox ();
        LoadButton = new Button ();
        ExpressionGB = new GroupBox ();
        Expression = new TextBox ();
        ExecuteButton = new Button ();
        XmlView = new TreeView ();

        // Initialize the controls
        Source.Anchor =
            AnchorStyles.Top | AnchorStyles.Left | AnchorStyles.Right;
        Source.Location = new System.Drawing.Point (16, 24);
        Source.Size = new System.Drawing.Size (336, 24);
        Source.TabIndex = 0;
        Source.Name = "Source";

        LoadButton.Anchor = AnchorStyles.Top | AnchorStyles.Right;
        LoadButton.Location = new System.Drawing.Point (368, 24);
        LoadButton.Size = new System.Drawing.Size (72, 24);
        LoadButton.TabIndex = 1;
        LoadButton.Text = "Load";
        LoadButton.Click += new System.EventHandler (OnLoadDocument);

        DocumentGB.Anchor =
            AnchorStyles.Top | AnchorStyles.Left | AnchorStyles.Right;
        DocumentGB.Location = new Point (16, 16);
        DocumentGB.Size = new Size (456, 64);
        DocumentGB.Text = "Document";
        DocumentGB.Controls.Add (Source);
        DocumentGB.Controls.Add (LoadButton);

        Expression.Anchor =
            AnchorStyles.Top | AnchorStyles.Left | AnchorStyles.Right;
        Expression.Location = new System.Drawing.Point (16, 24);
        Expression.Size = new System.Drawing.Size (336, 24);
        Expression.TabIndex = 2;
        Expression.Name = "Expression";

        ExecuteButton.Anchor = AnchorStyles.Top | AnchorStyles.Right;
        ExecuteButton.Location = new System.Drawing.Point (368, 24);
        ExecuteButton.Size = new System.Drawing.Size (72, 24);
        ExecuteButton.TabIndex = 3;
        ExecuteButton.Text = "Execute";
        ExecuteButton.Enabled = false;
        ExecuteButton.Click +=
            new System.EventHandler (OnExecuteExpression);

        ExpressionGB.Anchor =
            AnchorStyles.Top | AnchorStyles.Left | AnchorStyles.Right;
        ExpressionGB.Location = new System.Drawing.Point (16, 96);
        ExpressionGB.Name = "ExpressionGB";
        ExpressionGB.Size = new System.Drawing.Size (456, 64);
        ExpressionGB.Text = "Expression";
        ExpressionGB.Controls.Add (Expression);
        ExpressionGB.Controls.Add (ExecuteButton);

        NodeImages = new ImageList ();
        NodeImages.ImageSize = new Size (12, 12);
        NodeImages.Images.AddStrip (new Bitmap (GetType (), "Buttons"));
        NodeImages.TransparentColor = Color.White;

        XmlView.Anchor = AnchorStyles.Top | AnchorStyles.Bottom |
            AnchorStyles.Left | AnchorStyles.Right;
        XmlView.Location = new System.Drawing.Point (16, 176);
        XmlView.Size = new System.Drawing.Size (456, 232);
        XmlView.ImageList = NodeImages;
        XmlView.TabIndex = 4;
        XmlView.Name = "XmlView";

        // Add the controls to the form
        Controls.Add (DocumentGB);
        Controls.Add (ExpressionGB);
        Controls.Add (XmlView);
    }

    void OnLoadDocument (object sender, EventArgs e)
    {
        try {
            XPathDocument doc = new XPathDocument (Source.Text);
            Navigator = doc.CreateNavigator ();
            ExecuteButton.Enabled = true;
        }
        catch (Exception ex)  {
            MessageBox.Show (ex.Message);
        }
    }

    void OnExecuteExpression (object sender, EventArgs e)
    {
        try {
            XPathNodeIterator iterator =
                Navigator.Select (Expression.Text);
            XmlView.Nodes.Clear ();
            while (iterator.MoveNext ())
                AddNodeAndChildren (iterator.Current, null);
        }
        catch (Exception ex) {
            MessageBox.Show (ex.Message);
        }
    }

    void AddNodeAndChildren (XPathNavigator nav, TreeNode tnode)
    {
        TreeNode child = AddNode (nav, tnode);

        if (nav.HasAttributes) {
            nav.MoveToFirstAttribute ();
            do {
                AddAttribute (nav, child);
            } while (nav.MoveToNextAttribute ());
            nav.MoveToParent ();
        }

        if (nav.HasChildren) {
            nav.MoveToFirstChild ();
            do {
                AddNodeAndChildren (nav, child);
            } while (nav.MoveToNext ());
            nav.MoveToParent ();
        }
    }

    TreeNode AddNode (XPathNavigator nav, TreeNode tnode)
    {
        string text = null;
        TreeNode child = null;

        TreeNodeCollection tnodes = (tnode == null) ?
            XmlView.Nodes : tnode.Nodes;

        switch (nav.NodeType) {

        case XPathNodeType.Root:
        case XPathNodeType.Element:
            tnodes.Add (child = new TreeNode (nav.Name, 0, 0));
            break;

        case XPathNodeType.Attribute:
            text = String.Format ("{0}={1}", nav.Name, nav.Value);
            tnodes.Add (child = new TreeNode (text, 1, 1));
            break;

        case XPathNodeType.Text:
            text = nav.Value;
            if (text.Length > 128)
                text = text.Substring (0, 128) + "...";
            tnodes.Add (child = new TreeNode (text, 2, 2));
            break;

        case XPathNodeType.Comment:
            text = String.Format ("<!--{0}-->", nav.Value);
            tnodes.Add (child = new TreeNode (text, 4, 4));
            break;

        case XPathNodeType.ProcessingInstruction:
            text = String.Format ("<?{0} {1}?>", nav.Name, nav.Value);
            tnodes.Add (child = new TreeNode (text, 5, 5));
            break;
        }
        return child;
    }

    void AddAttribute (XPathNavigator nav, TreeNode tnode)
    {
        string text = String.Format ("{0}={1}", nav.Name, nav.Value);
        tnode.Nodes.Add (new TreeNode (text, 1, 1));
    }

    static void Main () 
    {
        Application.Run (new AnalyzerForm ());
    }
}