using System;
using System.Drawing;
using System.Windows.Forms;
using System.Collections;
using System.Threading;

class SieveForm : Form
{
    Label Label1;
    TextBox Input;
    TextBox Output;
    Button MyStartButton;
    Button MyCancelButton;
    Thread SieveThread;
    int MaxVal;

    SieveForm ()
    {
        // Initialize the form's properties
        Text = "MultiSieve";
        ClientSize = new System.Drawing.Size (292, 158);
        FormBorderStyle = FormBorderStyle.FixedDialog;
        MaximizeBox = false;

        // Instantiate the form's controls
        Label1 = new Label ();
        Input = new TextBox ();
        Output = new TextBox ();
        MyStartButton = new Button ();
        MyCancelButton = new Button ();

        // Initialize the controls
        Label1.Location = new Point (24, 28);
        Label1.Size = new Size (144, 16);
        Label1.Text = "Number of primes from 2 to";

        Input.Location = new Point (168, 24);
        Input.Size = new Size (96, 20);
        Input.Name = "Input";
        Input.TabIndex = 0;

        Output.Location = new Point (24, 64);
        Output.Size = new Size (240, 20);
        Output.Name = "Output";
        Output.ReadOnly = true;
        Output.TabStop = false;

        MyStartButton.Location = new Point (24, 104);
        MyStartButton.Size = new Size (104, 32);
        MyStartButton.Text = "Start";
        MyStartButton.TabIndex = 1;
        MyStartButton.Click += new EventHandler (OnStart);

        MyCancelButton.Location = new Point (160, 104);
        MyCancelButton.Size = new Size (104, 32);
        MyCancelButton.Text = "Cancel";
        MyCancelButton.TabIndex = 2;
        MyCancelButton.Enabled = false;
        MyCancelButton.Click += new EventHandler (OnCancel);

        // Add the controls to the form
        Controls.Add (Label1);
        Controls.Add (Input);
        Controls.Add (Output);
        Controls.Add (MyStartButton);
        Controls.Add (MyCancelButton);
    }

    void OnStart (object sender, EventArgs e)
    {
        // Get the number that the user typed
        try {
            MaxVal = Convert.ToInt32 (Input.Text);
        }
        catch (FormatException) {
            MessageBox.Show ("Please enter a number greater than 2");
            return;
        }

        if (MaxVal < 3) {
            MessageBox.Show ("Please enter a number greater than 2");
            return;
        }

        // Prepare the UI
        MyStartButton.Enabled = false;
        MyCancelButton.Enabled = true;
        Output.Text = "";

        // Start a background thread to count prime numbers
        SieveThread = new Thread (new ThreadStart (ThreadFunc));
        SieveThread.IsBackground = true;
        SieveThread.Start ();
    }

    void OnCancel (object sender, EventArgs e)
    {
        if (SieveThread != null && SieveThread.IsAlive) {
            // Terminate the background thread
            SieveThread.Abort ();

            // Wait until the thread terminates
            SieveThread.Join ();

            // Restore the UI
            MyStartButton.Enabled = true;
            MyCancelButton.Enabled = false;
            SieveThread = null;
        }
    }

    int CountPrimes (int max)
    {
        BitArray bits = new BitArray (max + 1, true);

        int limit = 2;
        while (limit * limit < max)
            limit++;

        for (int i=2; i<=limit; i++) {
            if (bits[i]) {
                for (int k=i + i; k<=max; k+=i)
                    bits[k] = false;
            }
        }

        int count = 0;
        for (int i=2; i<=max; i++) {
            if (bits[i])
                count++;
        }

        return count;
    }

    void ThreadFunc ()
    {
        // Do the computation
        int count = CountPrimes (MaxVal);

        // Update the UI
        Output.Text = count.ToString ();
        MyStartButton.Enabled = true;
        MyCancelButton.Enabled = false;
    }

    static void Main () 
    {
        Application.Run (new SieveForm ());
    }
}