using System;
using System.Threading;

class MyApp
{
    static Random rng = new Random ();
    static byte[] buffer = new byte[100];
    static Thread writer;
    static ReaderWriterLock rwlock = new ReaderWriterLock ();

    static void Main ()
    {
        // Initialize the buffer
        for (int i=0; i<100; i++)
            buffer[i] = (byte) (i + 1);

        // Start one writer thread
        writer = new Thread (new ThreadStart (WriterFunc));
        writer.Start ();

        // Start 10 reader threads
        Thread[] readers = new Thread[10];

        for (int i=0; i<10; i++) {
            readers[i] = new Thread (new ThreadStart (ReaderFunc));
            readers[i].Name = (i + 1).ToString ();
            readers[i].Start ();
        }
    }

    static void ReaderFunc ()
    {
        // Loop until the writer thread ends
        for (int i=0; writer.IsAlive; i++) {
            int sum = 0;

            // Sum the values in the buffer
            rwlock.AcquireReaderLock (Timeout.Infinite);

            try {
                for (int k=0; k<100; k++)
                    sum += buffer[k];
            }
            finally {
                rwlock.ReleaseReaderLock ();
            }

            // Report an error if the sum is incorrect
            if (sum != 5050) {
                string message = String.Format ("Thread {0} " +
                    "reports a corrupted read on iteration {1}",
                     Thread.CurrentThread.Name, i + 1);
                Console.WriteLine (message);
                writer.Abort ();
                return;
            }
        }
    }

    static void WriterFunc ()
    {
        DateTime start = DateTime.Now;

        // Loop for up to 10 seconds
        while ((DateTime.Now - start).Seconds < 10) {
            int j = rng.Next (0, 100);
            int k = rng.Next (0, 100);

            rwlock.AcquireWriterLock (Timeout.Infinite);

            try {
                Swap (ref buffer[j], ref buffer[k]);
            }
            finally {
                rwlock.ReleaseWriterLock ();
            }
        }
    }

    static void Swap (ref byte a, ref byte b)
    {
        byte tmp = a;
        a = b;
        b = tmp;
    }
}